# Copyright 2020 Askhat Omarov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from django.db import models
from django.utils.timezone import now

from .mixins import AbstractModelMixinTestCase
from ..models import Setting, update_timestamps, ensure_active


class SettingTestCase(AbstractModelMixinTestCase):
    mixin = Setting
    app_label = "dilcher_configuration"
    """
    This test case runs unit tests for Setting model
    """

    def test_model_fields(self):
        """
        This test ensures that correct fields are defined for the model
        """
        # 1. Prepare dictionary with expected fields and their parameters
        expected_fields = {
            'active': {
                'class': models.BooleanField,
                'default': False,
                'null': False,
                'max_length': None,
                'editable': True,
            },
            'name': {
                'class': models.CharField,
                'default': models.fields.NOT_PROVIDED,
                'null': False,
                'max_length': 254,
                'editable': True,
            },
            'last_activation_time': {
                'class': models.DateTimeField,
                'default': None,
                'null': True,
                'max_length': None,
                'editable': False,
            },
            'last_deactivation_time': {
                'class': models.DateTimeField,
                'default': None,
                'null': True,
                'max_length': None,
                'editable': False,
            },
            'last_value_change': {
                'class': models.DateTimeField,
                'default': None,
                'null': True,
                'max_length': None,
                'editable': False,
            },
        }

        # 2. Retrieve all fields of Settings model
        all_fields = Setting._meta.get_fields()

        # 3. Assert retrieved fields are only from expected_fields dictionary and are correctly defined
        self.assertEqual(len({x.name for x in all_fields}), len(expected_fields.keys()))
        for field in all_fields:
            self.assertTrue(field.name in expected_fields)
            self.assertTrue(isinstance(field, expected_fields.get(field.name)['class']))
            self.assertEqual(field.default, expected_fields.get(field.name)['default'])
            self.assertEqual(field.null, expected_fields.get(field.name)['null'])
            self.assertEqual(field.max_length, expected_fields.get(field.name)['max_length'])
            self.assertEqual(field.editable, expected_fields.get(field.name)['editable'])

    def test_model_default_fields(self):
        """
        This test checks default values that being set to a model instance
        """
        # 1. Create model instance without explicitly defined field values
        Model = self.__class__.model
        active_instance = Model.objects.create()
        result = Model.objects.create()

        # 2. Check automatically assigned values
        self.assertFalse(result.active)
        self.assertTrue(active_instance.active)
        self.assertEqual(result.name, '')
        self.assertEqual(result.last_activation_time, None)
        self.assertEqual(result.last_deactivation_time, None)
        self.assertIsNotNone(result.last_value_change)

        # 3. Clean up
        result.delete()
        active_instance.delete()

    def test_get_active_settings_empty(self):
        """
        When database does not store active settings, the function should return None
        """
        # 1. Execute function on model when database is empty and assert that it returned None
        Model = self.__class__.model
        result = Model.get_active_settings()
        self.assertIsNone(result)

        # 3. Execute function on model when database contains no models and assert that it returned None
        result = Model.get_active_settings()
        self.assertIsNone(result)

    def test_get_active_settings_with_active(self):
        """
        When database have active and inactive settings, only active one should be returned
        """
        # 1. Add active and inactive settings into database
        Model = self.__class__.model
        non_active = Model.objects.create(name='Test Inactive', active=False)
        active = Model.objects.create(name='Test Active', active=True)

        # 2. Execute function and assert that it returned an active instance
        result = Model.get_active_settings()
        self.assertEqual(result, active)

        # 3. Clean up
        active.delete()
        non_active.delete()

    def test___str___(self):
        """
        Check that function correctly returns a string representation of instance
        """
        # 1. Prepare datetime value and its string representation for usage in this test
        dt_now = now()
        str_dt_now = dt_now.strftime("%Y-%m-%d %H:%M:%S")

        # 2. Create inactive setting instance without 'last_activation_time' field
        Model = self.__class__.model
        non_active = Model.objects.create(name='Test Inactive', active=False)
        non_active.last_activation_time = None
        non_active.active = False
        # 3. Assert function result
        self.assertEqual(non_active.__str__(), 'Test Inactive (-)')

        # 4. Create inactive setting instance and with defined 'last_activation_time' field
        non_active_with_activation_time = Model.objects.create(
            name='Test Inactive 2',
            active=False,
            last_activation_time=dt_now,
        )
        non_active_with_activation_time.active = False
        non_active_with_activation_time.last_activation_time = dt_now
        # 5. Assert function result
        self.assertEqual(non_active_with_activation_time.__str__(), 'Test Inactive 2 ({0})'.format(str_dt_now))

        # 6. Create an active setting instance without 'last_activation_time' field
        active = Model.objects.create(name='Test Active', active=True)
        active.last_activation_time = None
        # 7. Assert function result
        self.assertEqual(active.__str__(), '* Test Active (-)')

        # 8. Create an active setting instance and with defined 'last_activation_time' field
        active_with_activation_time = Model.objects.create(
            name='Test Active 2',
            active=True,
            last_activation_time=dt_now,
        )
        active_with_activation_time.last_activation_time = dt_now
        # 9. Assert function result
        self.assertEqual(active_with_activation_time.__str__(), '* Test Active 2 ({0})'.format(str_dt_now))

        # 10. Clean up
        non_active.delete()
        non_active_with_activation_time.delete()
        active.delete()
        active_with_activation_time.delete()

    def test_update_timestamps_set_active_new_instance(self):
        """
        This test checks that correct timestamps are set for new active instance
        """
        # 1. Create active instance (but not save it to keep database empty) and prepare before_run timestamp
        Model = self.__class__.model
        before_run = now()
        instance = Model(active=True)

        # 2. Execute function
        update_timestamps(Model, instance)

        # 3. Prepare after_run timestamp
        after_run = now()

        # 4. Assert that 'last_activation_time' field is set, which is between 'before_run' and 'after_run' timestamp
        self.assertIsNotNone(instance.last_activation_time)
        self.assertTrue(instance.last_activation_time >= before_run)
        self.assertTrue(instance.last_activation_time <= after_run)

        # 5. Assert that 'last_deactivation_time' field is not set
        self.assertIsNone(instance.last_deactivation_time)

        # 6. Assert that 'last_value_change' field is set, which is between 'before_run' and 'after_run' timestamp
        self.assertIsNotNone(instance.last_value_change)
        self.assertTrue(instance.last_value_change >= before_run)
        self.assertTrue(instance.last_value_change <= after_run)

    def test_update_timestamps_set_active_updating_instance_from_inactive(self):
        """
        This test checks that correct timestamps applied when changing active flag from False to True
        """
        # 1. Create inactive instance in database and prepare before_run timestamp
        Model = self.__class__.model
        before_run = now()
        instance = Model.objects.create(active=False)
        instance.last_value_change = None

        # 2. Set instance to be active and execute function
        instance.active = True
        update_timestamps(Model, instance)

        # 3. Prepare after_run timestamp
        after_run = now()

        # 4. Assert that 'last_activation_time' field is set, which is between 'before_run' and 'after_run' timestamp
        self.assertIsNotNone(instance.last_activation_time)
        self.assertTrue(instance.last_activation_time >= before_run)
        self.assertTrue(instance.last_activation_time <= after_run)

        # 5. Assert that 'last_deactivation_time' and 'last_value_change' fields are not set
        self.assertIsNone(instance.last_deactivation_time)
        self.assertIsNone(instance.last_value_change)

        # 6. Clean up
        instance.delete()

    def test_update_timestamps_set_active_updating_instance_from_active(self):
        """
        This tests checks that nothing is done when 'active' flag is True between instances
        """
        # 1. Create active instance in database
        Model = self.__class__.model
        instance = Model.objects.create(active=True)
        instance.last_value_change = None

        # 2. Set instance to be active and execute function
        instance.active = True
        update_timestamps(Model, instance)

        # 3. Assert that timestamps were not assigned
        self.assertIsNotNone(instance.last_activation_time)
        self.assertIsNone(instance.last_deactivation_time)
        self.assertIsNone(instance.last_value_change)

        # 4. Clean up
        instance.delete()

    def test_update_timestamps_set_inactive_new_instance(self):
        """
        This test checks that correct timestamps are set for new inactive instance
        """
        # 1. Create inactive instance (but not save it to keep database empty) and prepare before_run timestamp
        Model = self.__class__.model
        before_run = now()
        instance = Model(active=False)

        # 2. Execute function
        update_timestamps(Model, instance)

        # 3. Prepare after_run timestamp
        after_run = now()

        # 4. Assert that 'last_activation_time' and 'last_deactivation_time' fields are not set
        self.assertIsNone(instance.last_activation_time)
        self.assertIsNone(instance.last_deactivation_time)

        # 5. Assert that 'last_value_change' field is set, which is between 'before_run' and 'after_run' timestamp
        self.assertIsNotNone(instance.last_value_change)
        self.assertTrue(instance.last_value_change >= before_run)
        self.assertTrue(instance.last_value_change <= after_run)

    def test_update_timestamps_set_inactive_updating_instance_from_active(self):
        """
        This test checks that correct timestamps applied when changing active flag from True to False
        """
        # 1. Create active instance in database and prepare before_run timestamp
        Model = self.__class__.model
        before_run = now()
        instance = Model.objects.create(active=True)
        instance.last_activation_time = None
        instance.last_value_change = None

        # 2. Set instance to be inactive and execute function
        instance.active = False
        update_timestamps(Model, instance)

        # 3. Prepare after_run timestamp
        after_run = now()

        # 4. Assert that 'last_deactivation_time' field is set, which is between 'before_run' and 'after_run' timestamp
        self.assertIsNotNone(instance.last_deactivation_time)
        self.assertTrue(instance.last_deactivation_time >= before_run)
        self.assertTrue(instance.last_deactivation_time <= after_run)

        # 5. Assert that 'last_activation_time' and 'last_value_change' fields are not set
        self.assertIsNone(instance.last_activation_time)
        self.assertIsNone(instance.last_value_change)

        # 6. Clean up
        instance.delete()

    def test_update_timestamps_set_inactive_updating_instance_from_inactive(self):
        """
        This test checks that nothing is done when 'active' flag is False between instances
        """
        # 1. Create inactive instance in database
        Model = self.__class__.model
        active_instance = Model.objects.create(active=True)
        instance = Model.objects.create(active=False)
        instance.last_value_change = None

        # 2. Set instance to be inactive and execute function
        instance.active = False
        update_timestamps(Model, instance)

        # 3. Assert that timestamps were not assigned
        self.assertIsNone(instance.last_activation_time)
        self.assertIsNone(instance.last_deactivation_time)
        self.assertIsNone(instance.last_value_change)

        # 4. Clean up
        instance.delete()
        active_instance.delete()

    def test_update_timestamps_value_change_excluded(self):
        """
        This test checks that 'last_value_change' field should not be updated for certain fields
        """
        # 1. Create Setting instance
        Model = self.__class__.model
        instance = Model.objects.create(active=False)
        instance.last_value_change = None
        initial_name = instance.name
        initial_active = instance.active
        initial_last_activation_time = instance.last_activation_time
        initial_last_deactivation_time = instance.last_deactivation_time

        # 2. Change instance 'name' value and assert that 'last_value_change' is unchanged after function execution
        instance.name = 'New name'
        update_timestamps(Model, instance)
        self.assertIsNone(instance.last_value_change)

        # 3. Change instance 'active' value and assert that 'last_value_change' is unchanged after function execution
        instance.active = not instance.active
        instance.name = initial_name
        update_timestamps(Model, instance)
        self.assertIsNone(instance.last_value_change)

        # 4. Change instance 'last_activation_time' value and assert that 'last_value_change' is unchanged after
        # function execution
        instance.active = initial_active
        instance.last_activation_time = now()
        update_timestamps(Model, instance)
        self.assertIsNone(instance.last_value_change)

        # 5. Change instance 'last_deactivation_time' value and assert that 'last_value_change' is unchanged after
        # function execution
        instance.last_activation_time = initial_last_activation_time
        instance.last_deactivation_time = now()
        update_timestamps(Model, instance)
        self.assertIsNone(instance.last_value_change)

        # 6. Change instance 'last_value_change' value and assert that 'last_value_change' is unchanged after
        # function execution
        instance.last_deactivation_time = initial_last_deactivation_time
        new_last_value_change = now()
        instance.last_value_change = new_last_value_change
        update_timestamps(Model, instance)
        self.assertIsNotNone(instance.last_value_change)
        self.assertEqual(instance.last_value_change, new_last_value_change)

        # 7. Clean up
        instance.delete()

    def test_update_timestamps_value_change(self):
        """
        This test checks that 'last_value_change' field should not be updated for certain fields
        """
        # 1. Create Setting instance and define 'before_run' timestamp
        Model = self.__class__.model
        instance = Model.objects.create(active=False)
        before_run = now()

        # 2. Change field value that is not included into excluded_names
        instance.id = 999999
        update_timestamps(Model, instance)

        # 3. Define 'after_run' timestamp
        after_run = now()

        # 4. Assert that instance has 'last_value_change' set and it is between 'before_run' and 'after_run' timestamps
        self.assertIsNotNone(instance.last_value_change)
        self.assertTrue(instance.last_value_change >= before_run)
        self.assertTrue(instance.last_value_change <= after_run)

        # 5. Clean up
        instance.delete()

    def test_update_timestamps_wrong_class(self):
        """
        Check that function accepts only Setting model
        """
        # 1. Prepare non-Setting class and its instance
        class Sample:
            pass
        instance = Sample()

        # 2. Assert that function with non-Setting class and instance raises TypeError
        with self.assertRaisesMessage(TypeError, 'Sender must be subclass of dilcher_configuration.models.Setting'):
            update_timestamps(Sample, instance)

    def test_ensure_active(self):
        """
        This function must deactivate other active instances, if active instance is passed as function parameter
        """
        # 1. Create active model instances
        Model = self.__class__.model
        instances = []
        for _ in range(10):
            instances.append(Model.objects.create(active=True))

        # 2. Create active control instance which will be passed to 'ensure_active' function
        control_instance = Model.objects.create(active=True)

        # 3. Execute function with 'control_instance' passed
        ensure_active(Model, control_instance)

        # 4. Assert that active instances now inactive
        for instance in instances:
            instance_in_db = Model.objects.get(pk=instance.pk)
            self.assertFalse(instance_in_db.active)

        # 5. Assert that control instance is still active
        control_instance_in_db = Model.objects.get(pk=control_instance.pk)
        self.assertTrue(control_instance_in_db.active)

        # 6. Clean up
        control_instance.delete()
        for instance in instances:
            instance.delete()

    def test_ensure_active_not_active_passed(self):
        """
        This function must leave active instances is is, if inactive instance is passed as function parameter
        """
        # 1. Create active model instances
        Model = self.__class__.model
        instances = []
        for _ in range(10):
            instances.append(Model.objects.create(active=True))

        # 2. Create inactive control instance which will be passed to 'ensure_active' function
        control_instance = Model.objects.create(active=False)

        # 3. Execute function with 'control_instance' passed
        ensure_active(Model, control_instance)

        # 4. Assert that previously active instances are all inactive - except for the last one
        for instance in instances:
            instance_in_db = Model.objects.get(pk=instance.pk)
            self.assertEqual(instance_in_db.active, instance_in_db.pk == instances[9].pk)

        # 5. Assert that control instance is still inactive
        control_instance_in_db = Model.objects.get(pk=control_instance.pk)
        self.assertFalse(control_instance_in_db.active)

        # 6. Clean up
        control_instance.delete()
        for instance in instances:
            instance.delete()

    def test_ensure_active_when_no_active_configs(self):
        """
        This function not activate inactive settings, if active instance is passed as function parameter
        """
        # 1. Create inactive model instances
        Model = self.__class__.model
        instances = []
        for _ in range(10):
            instances.append(Model.objects.create(active=False))

        # 2. Create active control instance which will be passed to 'ensure_active' function
        control_instance = Model.objects.create(active=True)

        # 3. Execute function with 'control_instance' passed
        ensure_active(Model, control_instance)

        # 4. Assert that inactive instances are still inactive
        for instance in instances:
            instance_in_db = Model.objects.get(pk=instance.pk)
            self.assertFalse(instance_in_db.active)

        # 5. Assert that control instance is still active
        control_instance_in_db = Model.objects.get(pk=control_instance.pk)
        self.assertTrue(control_instance_in_db.active)

        # 6. Clean up
        control_instance.delete()
        for instance in instances:
            instance.delete()

    def test_ensure_active_not_active_passed_no_active_configs(self):
        """
        This function not activate inactive settings, if inactive instance is passed as function parameter
        """
        # 1. Create inactive model instances
        Model = self.__class__.model
        instances = []
        for _ in range(10):
            instances.append(Model.objects.create(active=False))

        # 2. Create inactive control instance which will be passed to 'ensure_active' function
        control_instance = Model.objects.create(active=False)

        # 3. Execute function with 'control_instance' passed
        ensure_active(Model, control_instance)

        # 4. Assert that there is exactly one active instance
        self.assertEqual([Model.objects.get(pk=instance.pk).active for instance in instances].count(True), 1)

        # 5. Assert that control instance is still inactive
        control_instance_in_db = Model.objects.get(pk=control_instance.pk)
        self.assertFalse(control_instance_in_db.active)

        # 6. Clean up
        control_instance.delete()
        for instance in instances:
            instance.delete()

    def test_ensure_active_wrong_class(self):
        """
        Check that function accepts only Setting model
        """
        # 1. Prepare non-Setting class and its instance
        class Sample:
            pass
        instance = Sample()

        # 2. Assert that function with non-Setting class and instance raises TypeError
        with self.assertRaisesMessage(TypeError, 'Sender must be subclass of dilcher_configuration.models.Setting'):
            ensure_active(Sample, instance)



# Copyright 2020 Philipp Freyer
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

default_app_config = 'dilcher_configuration.apps.DilcherConfigurationConfig'


from django.db import models
from django.utils.timezone import now
from django.utils.translation import gettext_lazy as _


# Create your models here.
class Setting(models.Model):
    class Meta:
        abstract = True

    active = models.BooleanField(
        default=False,
        verbose_name=_("active"),
        help_text=_("""
Active flag. Set to True, if the current Settings instance should be considered as the active set of settings.
Only one instance may be active at any time. Any other instance will be set to inactive,
if the current instance is stored with active = True.
"""),
    )
    name = models.CharField(
        max_length=254,
        verbose_name=_("settings name"),
        help_text=_("""
The name of the Settings instance. Does not have any functionality,
this attribute is just used to describe and identify the current set of configuration variables.
"""),
    )
    last_activation_time = models.DateTimeField(
        verbose_name=_("time of last activation"),
        help_text=_(
            "Meta information. Set to the current date and time when a Settings instance is set to active=True."
        ),
        editable=False,
        null=True,
        default=None,
    )
    last_deactivation_time = models.DateTimeField(
        verbose_name=_("time of last deactivation"),
        help_text=_(
            "Meta information. Set to the current date and time when a Settings instance is set to active=False."
        ),
        editable=False,
        null=True,
        default=None,
    )
    last_value_change = models.DateTimeField(
        verbose_name=_("time of last data change"),
        help_text=_("""
Meta information. Set to the current date and time when a Settings instance is saved with changed configuration data.
"""
                    ),
        editable=False,
        null=True,
        default=None,
    )

    @classmethod
    def __init_subclass__(cls, **kwargs):
        super().__init_subclass__(**kwargs)
        models.signals.post_save.connect(ensure_active, sender=cls)
        models.signals.pre_save.connect(update_timestamps, sender=cls)

    @classmethod
    def get_active_settings(cls):
        """
        This method is used to easily retrieve the currently active settings object.
        This information is cached for a short amount of time (5 minutes) in order to
        speed up processing and reduce database load.
        Returns None, if no Setting object exists
        :return: Currently active settings object
        :rtype: Setting | None
        """
        try:
            return cls.objects.filter(active=True).first()
        except cls.DoesNotExist:
            return None

    def __str__(self):
        """
        This model renders as the following template:
        *{if active} <<name>> (<<last_activation_time>>)

        Example (active instance)
        * Current Settings (2020-01-20 17:03:42)

        Example (inactive instance)
        OldSettings (2020-01-20 17:03:40

        :return: String representation of the Settings instance
        :rtype: str
        """
        return "{active}{name} ({activation})".format(
            active="* " if self.active else "",
            name=self.name,
            activation=self.last_activation_time.strftime("%Y-%m-%d %H:%M:%S") if self.last_activation_time else "-"
        )


def update_timestamps(sender, instance, **_kwargs):
    """
    If the Setting model is saved, the meta data timestamps need to be updated:
    * Set the last_activation_time to now() if the active flag changed to True.
    * Set the last_deactivation_time to now() if the active flag changed to False.
    * Set the last_value_change to now() if any attribute that is not defined in the base class changes.

    :param sender: The sender class (Setting)
    :type sender: Type[Setting]
    :param instance: The setting instance that should be saved
    :type instance: Setting
    :param _kwargs: Additional keyword arguments
    """
    if not issubclass(sender, Setting):
        raise TypeError('Sender must be subclass of dilcher_configuration.models.Setting')

    try:
        old_instance = sender.objects.get(pk=instance.pk)
    except sender.DoesNotExist:
        old_instance = None

    # set last_activation_time, if required
    if instance.active and (old_instance is None or not old_instance.active):
        instance.last_activation_time = now()

    # set last_deactivation_time, if required
    if not instance.active and old_instance is not None and old_instance.active:
        instance.last_deactivation_time = now()

    # set last_value_change, if required
    if old_instance is None:
        instance.last_value_change = now()
    else:
        values_changed = False
        excluded_names = (
            "active",
            "name",
            "last_activation_time",
            "last_deactivation_time",
            "last_value_change",
        )
        for field in sender._meta.fields:
            if field.name not in excluded_names:
                if getattr(instance, field.name) != getattr(old_instance, field.name):
                    values_changed = True
                    break
        if values_changed:
            instance.last_value_change = now()


def ensure_active(sender, instance, **_kwargs):
    """
    Deactivate other Setting instances if the current instance has active set to True.

    :param sender: The sender class (Setting)
    :type sender: Type[Setting]
    :param instance: The setting instance that has been saved
    :type instance: Setting
    :param _kwargs: Additional keyword arguments
    """
    if not issubclass(sender, Setting):
        raise TypeError('Sender must be subclass of dilcher_configuration.models.Setting')

    if instance.active:
        for s in sender.objects.exclude(pk=instance.pk).filter(active=True):
            s.active = False
            s.save()
    else:
        if not sender.objects.filter(active=True).exists():
            new_active = sender.objects.exclude(pk=instance.pk).order_by("-last_activation_time").first()
            if new_active:
                new_active.active = True
                new_active.save()
            else:
                instance.active = True
                instance.save()



# Copyright 2020 Askhat Omarov
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from random import randint

from django.db import connection
from django.db.models.base import ModelBase
from django.db.utils import ProgrammingError
from django.test import TestCase


# Adapted from https://stackoverflow.com/questions/4281670/django-best-way-to-unit-test-an-abstract-model
class AbstractModelMixinTestCase(TestCase):
    """
    Base class for tests of model mixins. To use, subclass and specify the
    mixin class variable. A model using the mixin will be made available in
    self.model.
    This TestCase can be used for testing abstract classes
    """
    model = None
    mixin = None
    app_label = None

    @classmethod
    def setUpClass(cls) -> None:
        assert cls.mixin is not None, 'Define a `mixin` property to be able to define concrete model'

        class Meta():
            app_label = cls.app_label

        # Create a dummy model which extends the mixin. A RuntimeWarning will occur if the model is registered twice
        if cls.model is None:
            cls.model = ModelBase('__Model__' + str(randint(0, 100000000)) + '__' + cls.mixin.__name__,
                                  (cls.mixin,),
                                  {
                                      '__module__': cls.mixin.__module__,
                                      'Meta': Meta,
                                  })
            cls.enhance_model()

        # Create the schema for our test model. If the table already exists, will pass
        try:
            with connection.schema_editor() as schema_editor:
                schema_editor.create_model(cls.model)
            super().setUpClass()
        except ProgrammingError:
            pass

    @classmethod
    def tearDownClass(cls) -> None:
        # allow the transaction to exit
        super().tearDownClass()

        # Delete the schema for the test model. If no table, will pass
        try:
            with connection.schema_editor() as schema_editor:
                schema_editor.delete_model(cls.model)
        except ProgrammingError:
            pass

        connection.close()

    @classmethod
    def enhance_model(cls) -> None:
        """
        This method is used to add extra attributes to dummy model
        """



import setuptools

with open("Readme.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
  name="dilcher-configuration",       # Replace with your own username
  version="0.6.4",
  author="Philipp Freyer",
  author_email="philipp.freyer@dilcher.de",
  description="The dilcher-configuration Django app is used to provide easily manageable settings for Django projects.",
  long_description=long_description,
  long_description_content_type="text/markdown",
  url="https://github.com/cdilcher/dilcher-configuration",
  packages=setuptools.find_packages(),
  classifiers=[
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: Apache Software License",
    "Operating System :: OS Independent",
  ],
  python_requires='>=3.6',
  install_requires=[
    "Django>=3.0.0"
  ],
)
